
/*************************************************************************************************************************************

Questo sketch simula una tastiera USB HID (Human Interface Device) per inviare al pc una sequenza di tasti, definita dall'utente,

allo scopo di compilare automaticamente i form di accesso (login) ai siti Web che richiedono UserName e Password.

L'interfaccia utente  è ridotta ad un singolo pulsante che, se premuto brevemente e ripetutamente, permette di "scorrere" in sequenza

l'elenco dei siti che abbiamo memorizzato (vedi il file form_data), facendone apparire i nomi nel campo UserName. Arrivati a quello

desiderato, premendo il pulsante a lungo, verranno inseriti UserName e Password relativi, simulando anche lo spostamento tra i campi

(TAB) e l'invio (ENTER).

Questo progetto è stato inspirato dall'esempio "Trinket USB Keyboard" di Adafruit ed utilizza la relativa libreria "TrinketKeyboard" 

scaricabile dal sito ( https://learn.adafruit.com/trinket-usb-keyboard/overview ).

Sono stati utilizzati un microcontroller ATtiny85, Arduino IDE 1.6.12 per scheda "Adafruit Trinket 16MHz" e programmatore "USBasp"

Il settaggio dei fuses per l' ATtiny85 a 16MHz è: L:0xF1, H:0xD5, E:0xFE

****************************************************************************************************************************************/

#include <TrinketKeyboard.h>                     // include la libreria "TrinketKeyboard"
                                          
#define PIN_KEYBOARD_LAYOUT   0                  // definisce il pin collegato al jumper che attiva o meno il cambio di layout della tastiera
#define PIN_BUTTON_FILL_FORM  2                  // definisce il pin collegato al pulsante
#define PIN_LED               1                  // definisce il pin collegato al LED rosso che indica blocco o attività

//------------------------------- dichiarazione variabili globali ------------------------------------------------------------------

boolean lockedDevice = true;                     // memorizza lo stato del dispositivo (bloccato / sbloccato)  
byte unlockNumber = 0;                           // numero stabilito da raggiungere per lo sblocco
byte ledState = 0;                               // memorizza lo stato del LED Blocco Maiuscolo
byte ledStateLast = 0;                           // memorizza lo stato più recente del LED Blocco Maiuscolo
   
boolean buttonPressed = false;                   // memorizza lo stato del pusante (premuto / non premuto)
unsigned long buttonPressSeconds = 0;            // memorizza i secondi trascorsi dalla pressione del pulsante
byte buttonPressCounter = 0;                     // tiene il conto di quante volte è stato premuto il pulsante
byte buttonState = 0;                            // legge lo stato del pulsante (premuto / non premuto)

boolean formFilled = false;                      // memorizza l'ultima compilazione di un form

//------------------------------- definizioni stringhe tag e numero di form desiderati ----------------------------------------------
//---- E' necessario specificare il nome desiderato per il sito a cui accedere (tagStrings) ed il numero totale dei siti (tagsnumber). 

char* tagStrings[]={ "Elettronica & Maker", "eBay", "Subito.it", "PostePay", "Gmail (Chrome)" };  // puntatore ad Array di stringhe contenenti i tag
byte tagsNumber = 5;                                                                           // numero dei siti a cui si vuole accedere

//------------------------------- inizializzazione ----------------------------------------------------------------------------------

void setup(){
  
  pinMode(PIN_KEYBOARD_LAYOUT, INPUT_PULLUP);    // definisce il pin come input e attiva la resistenza interna di pull-up
  pinMode(PIN_BUTTON_FILL_FORM, INPUT_PULLUP);   // definisce il pin come input e attiva la resistenza interna di pull-up
  pinMode(PIN_LED, OUTPUT);                      // definisce il pin come output
  
  digitalWrite(PIN_LED, LOW);                    // imposta il LED rosso come spento

  TrinketKeyboard.begin();                       // inizializza le funzioni USB della libreria "TrinketKeyboard"
}

//------------------------------- inizio loop --------------------------------------------------------------------------------------

void loop(){

     unsigned long currentSeconds = millis()/1000;                                 // restituisce i secondi trascorsi dall'avvio del programma
  
     waitForUnlock();                                                              // attende lo sblocco del dispositivo
    
     TrinketKeyboard.poll();                                                       // funzione dellla libreria da richiamare periodicamnte per evitare errori nella connessione USB
  
     if (lockedDevice == false){                                                   // se il dispositivo è sbloccato

         buttonState = digitalRead(PIN_BUTTON_FILL_FORM);                          // legge lo stato del pulsante ad ogni ciclo di loop
      
         if (buttonState == LOW && buttonPressed == false){                        // se il pulsante passa da non premuto a premuto
      
             buttonPressSeconds = currentSeconds;                                  // memorizza quando è stato premuto
             
             buttonPressed = true;                                                 // evita ulteriori azionamenti

             formFilled = false;                                                   // segnala nessun form compilato
             
             digitalWrite(PIN_LED, HIGH);                                          // accende il LED rosso e spegne il blu
             
         }
         
         if (buttonState == HIGH && buttonPressed == true){                        // se il pulsante passa da premuto a non premuto

             if (formFilled == false){                                             // se nessun form è stato compilato
          
                if (buttonPressCounter == tagsNumber){buttonPressCounter = 0;}     // azzera il contatore delle pressioni se ha raggiunto il massimo
             
                delTag();                                                          // cancella eventuali dati presenti nella casella di testo
             
                typeTag(tagStrings[buttonPressCounter]);                           // digita la stringa del tag corrispondende al numero di pressioni
                                                                                   // passando la stringa alla funzione typeTag(char Tag[]) 
                delay(100);                                                        // ritardo per evitare rimbalzi del pulsante
             
                buttonPressCounter = buttonPressCounter + 1;                       // incrementa di uno il contatore delle pressioni

              }else{                                                               // se invece è stato compilato un form

                   buttonPressCounter = 0;                                         // azzera il contatore delle pressioni per ricominciare il ciclo
              }
         }

         if (buttonPressed == true && currentSeconds == buttonPressSeconds + 2){   // se il pulsante viene premuto per più di due secondi 

             delTag();                                                             // cancella il tag visibile nella casella di testo
             
             if (buttonPressCounter > 0){                                          // se il contatore è maggiore di zero
              
                 fillForm(buttonPressCounter);                                     // compila il form di numero corrispondente passando il
             }                                                                     // valore numerico alla funzione fillForm(byte formNumber)
                                                                                   // nel file form_data.ino
             buttonPressSeconds = 0;                                               // azzera la variabile del tempo trascorso
         }

         if (buttonState == HIGH){buttonPressed = false;}                          // se il pulsante non è premuto resetta la variabile di controllo
     }  
}

//------------------------------- funzione per il cambio layout di tastiera ----------------------------------------------------------------

void switchKeyboardLayout(){     
                                                     
     if (digitalRead(PIN_KEYBOARD_LAYOUT) == LOW){  // cambia il layout se il jumper J1 è chiuso, altrimenti lascia quello predefinito

         TrinketKeyboard.pressKey(KEYCODE_MOD_LEFT_ALT | KEYCODE_MOD_LEFT_SHIFT, 0);  // invia il comando ALT + SHIFT (cambio tra lingue di input in Windows)
     
         TrinketKeyboard.pressKey(0, 0);                                              // rilascia i tasti
     }   
}

//------------------------------- funzione per la digitazione dei tag----- ----------------------------------------------------------------

void typeTag(char Tag[]){

     switchKeyboardLayout();                                             // cambia il layout di tastiera se necessario
     
     TrinketKeyboard.print(Tag);                                         // digita la stringa tag tramite la funzione TrinketKeyboard.print della libreria "TrinketKeyboard" 

     switchKeyboardLayout();                                             // ripristina il layout di tastiera se necessario
    
     digitalWrite(PIN_LED, LOW);
}

//------------------------------- funzione per la cancellazione dei tag----- ----------------------------------------------------------------

void delTag(){

     TrinketKeyboard.pressKey(KEYCODE_MOD_LEFT_SHIFT, KEYCODE_HOME);      // invia il comando SHIFT + HOME per selezionare il testo da cancellare
     TrinketKeyboard.pressKey(0, 0);                                      // rilascia i tasti

     TrinketKeyboard.pressKey(0, KEYCODE_DELETE);                         // invia il comando DELETE per cancellare il testo           
     TrinketKeyboard.pressKey(0, 0);                                      // rilascia i tasti
}

//------------------------------- funzione per digitazione dello username e della password --------------------------------------------------

void typeString(char Str[]){

     TrinketKeyboard.print(Str);                                          // digita la stringa passata alla funzione
}

//------------------------------- funzione per il completamento del login -------------------------------------------------------------------

void pressENTER(){

     TrinketKeyboard.pressKey(0, KEYCODE_ENTER);                          // invia il comando ENTER per confermare il login           
     TrinketKeyboard.pressKey(0, 0);                                      // rilascia i tasti

     switchKeyboardLayout();                                              // ripristina il layout di tastiera se necessario

     formFilled = true;                                                   // conferma la compilazione del form

     digitalWrite(PIN_LED, LOW);                                          // spegne il LED rosso e accende il blu
}

//------------------------------- funzione per il passaggio al campo successivo -------------------------------------------------------------

void pressTAB(){

     TrinketKeyboard.pressKey(0, KEYCODE_TAB);                            // invia il comando TAB           
     TrinketKeyboard.pressKey(0, 0);                                      // rilascia i tasti
}

//------------------------------- funzione per lo sblocco di sicurezza ---------------------------------------------------------------------

void waitForUnlock(){

     if (lockedDevice == true){                           // se il dispositivo è bloccato (situazione iniziale)
     
         digitalWrite(PIN_LED, HIGH);                     // accende il LED rosso e spegne il blu

         ledStateLast = TrinketKeyboard.getLEDstate();    // memorizza lo stato più recente del LED BLOCCO MAIUSCOLO della tastiera, restituito dalla funzione TrinketKeyboard.getLEDstate()
         
         if (ledStateLast != ledState){                   // se lo stato è diverso dal precedente

             ledState = ledStateLast;                     // aggiorna la variabile relativa

             if (ledState == 2){                          // se lo stato vale 2 (solo LED BLOCCO MAIUSCOLO illuminato)

                 unlockNumber = unlockNumber + 2;         // incrementa di 2 unità il contatore del numero di sblocco

                 if (unlockNumber == 6){                  // se il contatore raggiunge 6 (LED BLOCCO MAIUSCOLO illuminato e spento 3 volte)

                     lockedDevice = false;                // variabile di sblocco resettata, dispositivo attivo
                     digitalWrite(PIN_LED, LOW);          // spegne LED rosso, accende LED blu
                }
            }
        }  
     }
}



